@@ -25,7 +25,6 @@
 # CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
 """
 A pure Python implementation of the OPeNDAP server protocol.
 
@@ -42,20 +41,27 @@ clients using the netCDF4 library. PyDAP client libraries are not supported.
 """
 
 import re
+from dataclasses import dataclass
+
+import dask.array as da
 import numpy as np
 
 INDENT = '    '
 SLICE_CONSTRAINT_RE = r'\[([\d,\W]+)\]$'
 
 
+@dataclass
+class Config:
+    DASK_ENCODE_CHUNK_SIZE: int = 20e6
+
+
 class DAPError(Exception):
     pass
 
 
-class DAPObject(object):
+class DAPObject:
     """A generic DAP object class.
     """
-
     def __init__(self, name='', parent=None, *args, **kwargs):
         try:
             self.name = '_'.join(name.split(' '))
@@ -132,12 +138,12 @@ class DAPObject(object):
                 return '.'.join([self.parent.data_path, self.name])
 
     def ddshead(self):
-        return '{indent}{obj} {{\n'.format(
-            indent=self.indent, obj=self.__class__.__name__)
+        return '{indent}{obj} {{\n'.format(indent=self.indent,
+                                           obj=self.__class__.__name__)
 
     def ddstail(self):
-        return '{indent}}} {name};\n'.format(
-            indent=self.indent, name=self.name)
+        return '{indent}}} {name};\n'.format(indent=self.indent,
+                                             name=self.name)
 
     def dashead(self):
         name = self.name
@@ -211,12 +217,13 @@ class DAPAtom(DAPObject):
 
     def dds(self, constraint=''):
         if meets_constraint(constraint, self.data_path):
-            yield '{indent}{dtype} {name};\n'.format(
-                indent=self.indent, dtype=self.__str__(), name=self.name)
+            yield '{indent}{dtype} {name};\n'.format(indent=self.indent,
+                                                     dtype=self.__str__(),
+                                                     name=self.name)
 
     def dods_data(self, constraint=''):
         if meets_constraint(constraint, self.data_path):
-            yield dods_encode(self._val, self)
+            yield from dods_encode(self._val, self)
 
 
 class Byte(DAPAtom):
@@ -226,45 +233,45 @@ class Byte(DAPAtom):
 
 class Int16(DAPAtom):
     dtype = np.int16
-    str = '<i4'
+    str = '>i4'
 
 
 class UInt16(DAPAtom):
     dtype = np.uint16
-    str = '<u4'
+    str = '>u4'
 
 
 class Int32(DAPAtom):
     dtype = np.int32
-    str = '<i4'
+    str = '>i4'
 
 
 class UInt32(DAPAtom):
     dtype = np.uint32
-    str = '<u4'
+    str = '>u4'
 
 
 class Float32(DAPAtom):
     dtype = np.float32
-    str = '<f4'
+    str = '>f4'
 
 
 class Float64(DAPAtom):
     dtype = np.float64
-    str = '<f8'
+    str = '>f8'
 
 
 class String(DAPAtom):
-    dtype = np.str
+    dtype = np.str_
     str = 'S'
 
     def dods_data(self, constraint=''):
         if meets_constraint(constraint, self.data_path):
-            yield dods_encode(self._val.encode('ascii'), self)
+            yield from dods_encode(self._val.encode('ascii'), self)
 
 
 class URL(String):
-    dtype = np.str
+    dtype = np.str_
     str = 'S'
 
 
@@ -277,7 +284,6 @@ class Structure(DAPObject):
 class Dataset(Structure):
     """Class representing a DAP dataset.
     """
-
     def dods_data(self, constraint=''):
 
         yield b'Data:\r\n'
@@ -342,7 +348,6 @@ class Sequence(DAPObject):
 class SequenceInstance(DAPObject):
     """Class representing a data item that will be added to a sequence.
     """
-
     @property
     def data_path(self):
         return self.parent.data_path
@@ -372,7 +377,6 @@ class DAPDataObject(DAPObject):
     """A generic class for typed non-atomic objects holding actual data (i.e.
     Array and Grid).
     """
-
     def _parse_args(self, args, kwargs):
 
         self.data = kwargs.get('data', None)
@@ -394,11 +398,11 @@ class DAPDataObject(DAPObject):
 
         if meets_constraint(constraint, self.data_path):
             slices = parse_slice_constraint(constraint)
-            yield dods_encode(self.data[slices], self.dtype)
+            yield from dods_encode(self.data[slices], self.dtype)
             if self.dimensions is not None:
                 for i, dim in enumerate(self.dimensions):
                     sl = slices[i] if i < len(slices) else ...
-                    yield dods_encode(dim.data[sl], dim.dtype)
+                    yield from dods_encode(dim.data[sl], dim.dtype)
 
 
 class Grid(DAPDataObject):
@@ -482,9 +486,18 @@ def dods_encode(data, dtype):
     if not is_scalar:
         packed_length = length.astype('<i4').byteswap().tobytes() * 2
 
-    packed_data = data.astype(dtype.str).byteswap().tobytes()
+    yield packed_length
 
-    return packed_length + packed_data
+    if isinstance(data, da.Array):
+        # Encode in chunks of a defined size if we work with dask.Array
+        chunk_size = int(Config.DASK_ENCODE_CHUNK_SIZE / data.dtype.itemsize)
+        serialize_data = data.ravel().rechunk(chunk_size)
+        for block in serialize_data.blocks:
+            yield block.astype(dtype.str).compute().tobytes()
+    else:
+        # Make sure we always encode an array or we will get wrong results
+        data = np.asarray(data)
+        yield data.astype(dtype.str).tobytes()
 
 
 def parse_slice_constraint(constraint):
@@ -539,3 +552,16 @@ def meets_constraint(constraint_expr, data_path):
             return True
 
     return False
+
+
+def set_dask_encoding_chunk_size(chunk_size: int):
+    """Set the maximum chunk size used to encode ``dask.Array``s to XDR.
+
+    :param chunk_size: (int) Encoding chunk size in Bytes
+    :returns: None
+    """
+    chunk_size = int(chunk_size)
+    if chunk_size > 0: 
+        Config.DASK_ENCODE_CHUNK_SIZE = chunk_size
+    else:
+        raise ValueError('Encoding chunk size needs to be greather than 0.')
